
SET client_encoding = 'SQL_ASCII';
SET check_function_bodies = false;
CREATE SCHEMA @@SCHEMA@@;
SET search_path = @@SCHEMA@@, pg_catalog;
CREATE TABLE cv_interest (
    pint_class character varying(1024) NOT NULL,
    pint_order integer DEFAULT 2147483647 NOT NULL,
    weight integer,
    CONSTRAINT cv_pint_class_chk CHECK (((pint_class)::text ~ '^[a-z]+$'::text))
) WITHOUT OIDS;

CREATE TABLE prg_interest (
    prg_title text NOT NULL,
    pint_class character varying(1024) NOT NULL
) WITHOUT OIDS;

CREATE TABLE bayes_toks (
    prg_oid bigint,
    prg_title text,
    tok_src character(1) NOT NULL,
    tok_name character varying(50) NOT NULL,
    tok_subname character varying(50),
    tok_value text NOT NULL,
    pint_class character varying(1024) NOT NULL,
    CONSTRAINT bayes_toks_tok_src CHECK (((tok_src = 'p'::bpchar) OR (tok_src = 'c'::bpchar)))
) WITHOUT OIDS;

CREATE FUNCTION longer(character varying, character varying) RETURNS character varying
    AS 'select case when length($2) > length($1) then $2 else $1 end;'
    LANGUAGE sql IMMUTABLE;

CREATE AGGREGATE longest (
    BASETYPE = character varying,
    SFUNC = longer,
    STYPE = character varying,
    INITCOND = ''
);

CREATE FUNCTION round_5min(timestamp with time zone) RETURNS timestamp with time zone
    AS 'DECLARE
	result timestamptz;
	fixmin int4;
BEGIN
	result := $1;
	result := date_trunc(''minute'', result);
	fixmin := extract(minute from result);
	fixmin := fixmin % 5;
	result := result - (fixmin || '' minutes'')::interval;
	return result;
END;'
    LANGUAGE plpgsql STABLE;

CREATE FUNCTION now_5min() RETURNS timestamp with time zone
    AS 'select @@SCHEMA@@.round_5min(now());'
    LANGUAGE sql STABLE;

CREATE FUNCTION grid_end() RETURNS timestamp with time zone
    AS 'select @@SCHEMA@@.grid_endx(@@SCHEMA@@.now_5min())'
    LANGUAGE sql STABLE;

CREATE FUNCTION roundup_5min(timestamp with time zone) RETURNS timestamp with time zone
    AS 'DECLARE
	result timestamptz;
	fixmin int4;
BEGIN
	result := $1;
	result := date_trunc(''minute'', result);
	fixmin := extract(minute from result);
	fixmin := fixmin % 5;
	if fixmin > 0 then
		result := result + ((5 - fixmin) || '' minutes'')::interval;
	end if;
	return result;
END;'
    LANGUAGE plpgsql STABLE;

CREATE FUNCTION blk_start(timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 < @@SCHEMA@@.now_5min()
then @@SCHEMA@@.now_5min()
else @@SCHEMA@@.round_5min($1) end'
    LANGUAGE sql STABLE;

CREATE FUNCTION blk_end(timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 > @@SCHEMA@@.grid_end() then @@SCHEMA@@.grid_end() else @@SCHEMA@@.roundup_5min($1) end'
    LANGUAGE sql STABLE;

CREATE TABLE channels (
    chan_oid bigserial NOT NULL,
    chan_id character varying(1024) NOT NULL
) WITHOUT OIDS;

CREATE TABLE chan_attrs (
    catt_oid bigserial NOT NULL,
    chan_oid bigint NOT NULL,
    catt_name character varying(50) NOT NULL,
    catt_value text NOT NULL
) WITHOUT OIDS;

CREATE TABLE icons (
    icn_oid bigserial NOT NULL,
    icn_src character varying(1024) NOT NULL,
    icn_width integer,
    icn_height integer
) WITHOUT OIDS;

CREATE TABLE chan_icons (
    icn_oid bigint,
    icn_src character varying(1024),
    icn_width integer,
    icn_height integer,
    chan_oid bigint NOT NULL
)
INHERITS (icons) WITHOUT OIDS;

CREATE TABLE programmes (
    prg_oid bigserial NOT NULL,
    prg_start timestamp with time zone NOT NULL,
    prg_stop timestamp with time zone,
    chan_oid bigint NOT NULL
) WITHOUT OIDS;

CREATE TABLE prg_icons (
    icn_oid bigint,
    icn_src character varying(1024),
    icn_width integer,
    icn_height integer,
    prg_oid bigint NOT NULL
)
INHERITS (icons) WITHOUT OIDS;

CREATE TABLE prg_attrs (
    patt_oid bigserial NOT NULL,
    prg_oid bigint NOT NULL,
    patt_name character varying(50) NOT NULL,
    patt_subname character varying(50),
    patt_lang character varying(2),
    patt_value text NOT NULL
) WITHOUT OIDS;

CREATE TABLE patt_icons (
    icn_oid bigint,
    icn_src character varying(1024),
    icn_width integer,
    icn_height integer,
    patt_oid bigint NOT NULL
)
INHERITS (icons) WITHOUT OIDS;

CREATE VIEW chan_best_name AS
    SELECT chan_attrs.chan_oid, ("substring"(chan_attrs.catt_value, '^\\d+'::text))::integer AS chan_number, "substring"(chan_attrs.catt_value, '^\\d+ (.*)$'::text) AS chan_name FROM chan_attrs WHERE ((chan_attrs.catt_name)::text = '_number_name'::text);

CREATE FUNCTION grid_endx(timestamp with time zone) RETURNS timestamp with time zone
    AS 'DECLARE
	res timestamptz;
	tmins int4;
BEGIN
	res := @@SCHEMA@@.roundup_5min($1);
	return res + ''2 hours''::interval;
END;'
    LANGUAGE plpgsql STABLE;

CREATE FUNCTION blk_startx(timestamp with time zone, timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 < @@SCHEMA@@.round_5min($2)
then @@SCHEMA@@.round_5min($2)
else @@SCHEMA@@.round_5min($1) end'
    LANGUAGE sql STABLE;

CREATE FUNCTION prgs_then(timestamp with time zone) RETURNS SETOF programmes
    AS 'SELECT *
   FROM @@SCHEMA@@.programmes p
WHERE p.prg_start >= ($1 - ''08:00:00''::interval)
AND p.prg_start < @@SCHEMA@@.grid_endx($1)
AND p.prg_stop > @@SCHEMA@@.round_5min($1)
AND p.prg_stop <= (@@SCHEMA@@.grid_endx($1) + ''08:00:00''::interval)'
    LANGUAGE sql STABLE;

CREATE FUNCTION prg_attrs_then(timestamp with time zone) RETURNS SETOF prg_attrs
    AS 'SELECT pa.* 
   FROM @@SCHEMA@@.prg_attrs pa
   natural join @@SCHEMA@@.programmes p
WHERE p.prg_start >= ($1 - ''08:00:00''::interval)
AND p.prg_start < @@SCHEMA@@.grid_endx($1)
AND p.prg_stop > @@SCHEMA@@.round_5min($1)
AND p.prg_stop <= (@@SCHEMA@@.grid_endx($1) + ''08:00:00''::interval)'
    LANGUAGE sql STABLE;

CREATE FUNCTION agg_prod_next(double precision, double precision) RETURNS double precision
    AS 'BEGIN
	return $1 * $2;
END;
'
    LANGUAGE plpgsql IMMUTABLE;

CREATE AGGREGATE product (
    BASETYPE = double precision,
    SFUNC = agg_prod_next,
    STYPE = double precision,
    INITCOND = '1.0'
);

CREATE FUNCTION tokenizer(text) RETURNS SETOF text
    AS 'DECLARE
	splitme text;	
	word text;
BEGIN
	splitme := $1;
	WHILE length(splitme) > 0 LOOP
		-- trim leading junk
		splitme := substring(splitme from ''^\\\\W*(.*)'');
		-- snarf a word
		word := substring(splitme from ''^(\\\\w*)'');
		if length(word) > 0 then
			return next word;
		end if;
		-- trim leading word
		splitme := substring(splitme from ''^\\\\w*(.*)'');
	END LOOP;
	return;
END;'
    LANGUAGE plpgsql IMMUTABLE STRICT;

CREATE TABLE tok_counts_val_pint_mv (
    tok_src character(1),
    tok_name character varying(50),
    tok_subname character varying,
    tok_value text,
    pint_class character varying(1024),
    count_val_pint bigint
);

CREATE TABLE tok_counts_val_mv (
    tok_src character(1),
    tok_name character varying(50),
    tok_subname character varying,
    tok_value text,
    count_val bigint
);

CREATE TABLE tok_counts_pint_mv (
    pint_class character varying(1024),
    count_pint bigint
);

CREATE TABLE tok_counts_full_mv (
    tok_src character(1),
    tok_name character varying(50),
    tok_subname character varying,
    tok_value text,
    pint_class character varying(1024),
    count_val_pint bigint,
    count_val bigint,
    count_pint bigint,
    p_pint double precision
);

CREATE FUNCTION get_bayes_class(bigint) RETURNS character varying
    AS 'DECLARE
	prgoid alias for $1;
	crow @@SCHEMA@@.bayes_cache%ROWTYPE;
BEGIN
	select into crow * from @@SCHEMA@@.bayes_cache where prg_oid = prgoid;
	if crow.pint_class is not null and now() - crow.lastupdate > ''24 hours''::interval then
		delete from @@SCHEMA@@.bayes_cache where prg_oid = prgoid;
		crow.pint_class := null;
	end if;
	if crow.pint_class is null then
		select into crow * from @@SCHEMA@@.compute_bayes_class(prgoid);
		insert into @@SCHEMA@@.bayes_cache (prg_oid, prg_title, pint_class,
				bayes_prob, lastupdate, num_toks)
			values (crow.prg_oid, crow.prg_title, crow.pint_class,
				crow.bayes_prob, crow.lastupdate, crow.num_toks);
	end if;
	return crow.pint_class;
END;
'
    LANGUAGE plpgsql STABLE STRICT;

CREATE FUNCTION get_prg_toks(bigint, character varying, character varying) RETURNS SETOF bayes_toks
    AS 'DECLARE
	prgoid alias for $1;
	prgtitle alias for $2;
	pintclass alias for $3;
	outrow @@SCHEMA@@.bayes_toks%ROWTYPE;
	pattrow @@SCHEMA@@.prg_attrs%ROWTYPE;
	cattrow @@SCHEMA@@.chan_attrs%ROWTYPE;
	tok record;
BEGIN
	if prgtitle is null then
		raise exception ''prg_title must not be null for prg_oid %'', prgoid;
	end if;
	if prgoid is null or pintclass is null then
		return;
	end if;
	outrow.prg_oid = prgoid;
	outrow.prg_title = prgtitle;
	outrow.pint_class = pintclass;
	outrow.tok_src := ''p'';
	-- tokenize programme attributes
	for pattrow in select * from @@SCHEMA@@.prg_attrs where prg_oid = outrow.prg_oid loop
		outrow.tok_name := pattrow.patt_name;
		outrow.tok_subname := coalesce(pattrow.patt_subname, '''');
		for tok in select distinct lower(t) as t
				from @@SCHEMA@@.tokenizer(pattrow.patt_value) as t
				where length(t) > 3 loop
			outrow.tok_value = tok.t;
			return next outrow;
		end loop;
	end loop;
	-- tokenize channel attributes
	outrow.tok_src := ''c'';
	outrow.tok_subname := null;
	for cattrow in select ca.* from @@SCHEMA@@.chan_attrs ca
			inner join @@SCHEMA@@.programmes p on p.chan_oid = ca.chan_oid
			where p.prg_oid = outrow.prg_oid loop
		outrow.tok_name := cattrow.catt_name;
		for tok in select distinct lower(t) as t
				from @@SCHEMA@@.tokenizer(cattrow.catt_value) as t
				where length(t) > 3 loop
			outrow.tok_value = tok.t;
			return next outrow;
		end loop;
	end loop;
	return;
END;'
    LANGUAGE plpgsql STABLE;

CREATE FUNCTION bayes_learn_prg(bigint, character varying, character varying) RETURNS integer
    AS 'DECLARE
	prgoid alias for $1;
	prgtitle varchar;
	pintclass alias for $3;
	learnpintclass varchar;
	learnprgtitle varchar;
	rowcount int4;
BEGIN
	if prgoid is null then
		raise exception ''prgoid must not be null'';
	end if;
	delete from @@SCHEMA@@.bayes_toks where prg_oid = prgoid;
	if pintclass is null then
		return 0;
	end if;
	prgtitle := coalesce($2, '''');
	-- any lookups to do?
	if length(pintclass) = 0 or length(prgtitle) = 0 then
		select into learnprgtitle, learnpintclass longest_prg_title, pint_class
			from @@SCHEMA@@.prg_long_title left join @@SCHEMA@@.prg_interest
			on longest_prg_title = prg_title
			where prg_oid = prgoid;
	end if;
	if length(pintclass) > 0 then
		learnpintclass := coalesce(pintclass, '''');
	end if;
	if length(prgtitle) > 0 then
		learnprgtitle := coalesce(prgtitle, '''');
	end if;
	if length(learnpintclass) = 0 then
		raise exception ''no auto pint_class found for prg_oid %'', prgoid;
	end if;
	if length(learnprgtitle) = 0 then
		raise exception ''no auto prg_title found ofr prg_oid %'', prgoid;
	end if;
	insert into @@SCHEMA@@.bayes_toks
		select distinct *
		from @@SCHEMA@@.get_prg_toks(prgoid, learnprgtitle, learnpintclass);
	get diagnostics rowcount = ROW_COUNT;
	return rowcount;
END;
'
    LANGUAGE plpgsql;

CREATE FUNCTION min(anyelement, anyelement) RETURNS anyelement
    AS 'BEGIN
	if $1 < $2 then
		return $1;
	else
		return $2;
	end if;
end;
'
    LANGUAGE plpgsql IMMUTABLE STRICT;

CREATE FUNCTION max(anyelement, anyelement) RETURNS anyelement
    AS 'BEGIN
	if $1 > $2 then
		return $1;
	else
		return $2;
	end if;
end;
'
    LANGUAGE plpgsql IMMUTABLE STRICT;

CREATE TABLE tok_stats_mv (
    tok_src character(1),
    tok_name character varying(50),
    tok_subname character varying,
    tok_value text,
    pint_class character varying(1024),
    tok_val_pint_ness double precision
);

CREATE FUNCTION blk_endx(timestamp with time zone, timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 > @@SCHEMA@@.grid_endx($2)
then @@SCHEMA@@.grid_endx($2)
else @@SCHEMA@@.roundup_5min($1) end'
    LANGUAGE sql STABLE;

CREATE VIEW prgs_now AS
    SELECT programmes.prg_oid, programmes.prg_start, programmes.prg_stop, programmes.chan_oid FROM programmes WHERE (((programmes.prg_start >= (now() - '08:00:00'::interval)) AND (programmes.prg_start <= now())) AND (programmes.prg_stop > now()));

CREATE TABLE tok_sum_pint_mv (
    tok_src character(1),
    tok_name character varying(50),
    tok_subname character varying,
    tok_value text,
    sum_p_pint double precision
);

CREATE FUNCTION update_bayes_stats() RETURNS void
    AS 'DECLARE
	bt_corr real;
BEGIN
	-- check if we should cluster our toks table
	analyze @@SCHEMA@@.bayes_toks;
	select into bt_corr correlation
		from pg_catalog.pg_stats
		where schemaname = ''@@SCHEMA@@'' and tablename = ''bayes_toks''
		and attname = ''tok_src'';
	if bt_corr < 0.9 then
		cluster @@SCHEMA@@.bayes_toks;
	end if;
	-- flush our matviews
	delete from @@SCHEMA@@.tok_stats_mv;
	delete from @@SCHEMA@@.tok_sum_pint_mv;
	delete from @@SCHEMA@@.tok_counts_full_mv;
	delete from @@SCHEMA@@.tok_counts_pint_mv;
	delete from @@SCHEMA@@.tok_counts_val_mv;
	delete from @@SCHEMA@@.tok_counts_val_pint_mv;
	-- repopuplate our matviews
	insert into @@SCHEMA@@.tok_counts_val_pint_mv select * from @@SCHEMA@@.tok_counts_val_pint;
	insert into @@SCHEMA@@.tok_counts_val_mv select * from @@SCHEMA@@.tok_counts_val;
	insert into @@SCHEMA@@.tok_counts_pint_mv select * from @@SCHEMA@@.tok_counts_pint;
	insert into @@SCHEMA@@.tok_counts_full_mv select * from @@SCHEMA@@.tok_counts_full;
	insert into @@SCHEMA@@.tok_sum_pint_mv select * from @@SCHEMA@@.tok_sum_pint;
	insert into @@SCHEMA@@.tok_stats_mv select * from @@SCHEMA@@.tok_stats;
	-- re-cluster and analyze the new data
	cluster @@SCHEMA@@.tok_stats_mv;
	analyze @@SCHEMA@@.tok_stats_mv;

	delete from @@SCHEMA@@.bayes_cache;
	
	return;
end;'
    LANGUAGE plpgsql;

CREATE VIEW tok_counts_val_pint AS
    SELECT bayes_toks.tok_src, bayes_toks.tok_name, COALESCE(bayes_toks.tok_subname, ''::character varying) AS tok_subname, bayes_toks.tok_value, bayes_toks.pint_class, count(DISTINCT bayes_toks.prg_oid) AS count_val_pint FROM bayes_toks GROUP BY bayes_toks.tok_src, bayes_toks.tok_name, bayes_toks.tok_subname, bayes_toks.tok_value, bayes_toks.pint_class;

CREATE VIEW tok_counts_val AS
    SELECT tok_counts_val_pint_mv.tok_src, tok_counts_val_pint_mv.tok_name, COALESCE(tok_counts_val_pint_mv.tok_subname, ''::character varying) AS tok_subname, tok_counts_val_pint_mv.tok_value, (sum(tok_counts_val_pint_mv.count_val_pint))::bigint AS count_val FROM tok_counts_val_pint_mv GROUP BY tok_counts_val_pint_mv.tok_src, tok_counts_val_pint_mv.tok_name, tok_counts_val_pint_mv.tok_subname, tok_counts_val_pint_mv.tok_value;

CREATE VIEW tok_counts_pint AS
    SELECT bayes_toks.pint_class, count(DISTINCT bayes_toks.prg_oid) AS count_pint FROM bayes_toks GROUP BY bayes_toks.pint_class;

CREATE VIEW tok_counts_full AS
    SELECT tok_src, tok_name, tok_subname, tok_value, pint_class, count_val_pint, count_val, count_pint, ((count_val_pint)::double precision / (count_pint)::double precision) AS p_pint FROM ((tok_counts_val_pint_mv NATURAL JOIN tok_counts_val_mv) NATURAL JOIN tok_counts_pint_mv);

CREATE VIEW tok_sum_pint AS
    SELECT tok_counts_full_mv.tok_src, tok_counts_full_mv.tok_name, tok_counts_full_mv.tok_subname, tok_counts_full_mv.tok_value, sum(tok_counts_full_mv.p_pint) AS sum_p_pint FROM tok_counts_full_mv GROUP BY tok_counts_full_mv.tok_src, tok_counts_full_mv.tok_name, tok_counts_full_mv.tok_subname, tok_counts_full_mv.tok_value;

CREATE VIEW tok_stats AS
    SELECT tok_counts_full_mv.tok_src, tok_counts_full_mv.tok_name, tok_counts_full_mv.tok_subname, tok_counts_full_mv.tok_value, pint_class, min((0.99)::double precision, max((0.01)::double precision, (p_pint / sum_p_pint))) AS tok_val_pint_ness FROM (tok_counts_full_mv NATURAL JOIN tok_sum_pint_mv);

CREATE VIEW prg_long_title AS
    SELECT prg_attrs.prg_oid, prg_attrs.patt_value AS longest_prg_title FROM prg_attrs WHERE ((prg_attrs.patt_name)::text = 'title'::text);

CREATE VIEW shows AS
    SELECT prg_oid, prg_start, prg_stop, prg_long_title.longest_prg_title AS prg_title, chan_oid, chan_number, chan_name FROM (((programmes NATURAL JOIN channels) NATURAL JOIN chan_best_name) NATURAL JOIN prg_long_title);

CREATE VIEW chan_quality AS
    SELECT cq.chan_name, cq.pint_class, cq.totaltime, (cq.totaltime * (weight)::double precision) AS weightedtime FROM ((SELECT chan_name, COALESCE(pint_class, 'normal'::character varying) AS pint_class, sum((prg_stop - prg_start)) AS totaltime FROM (shows NATURAL LEFT JOIN prg_interest) GROUP BY chan_name, COALESCE(pint_class, 'normal'::character varying)) cq NATURAL JOIN cv_interest) ORDER BY cq.chan_name, cq.pint_class;

CREATE TABLE bayes_cache (
    prg_oid bigint NOT NULL,
    prg_title text NOT NULL,
    pint_class character varying(1024) NOT NULL,
    bayes_prob double precision NOT NULL,
    lastupdate timestamp with time zone DEFAULT now() NOT NULL,
    num_toks bigint NOT NULL
) WITHOUT OIDS;

CREATE FUNCTION compute_bayes_class_all(bigint) RETURNS SETOF bayes_cache
    AS 'select pt.prg_oid, pt.prg_title, ts.pint_class,
	@@SCHEMA@@.min(0.9999::float8, @@SCHEMA@@.product(ts.tok_val_pint_ness) /
	(@@SCHEMA@@.product(ts.tok_val_pint_ness)
	+ @@SCHEMA@@.product(1 - ts.tok_val_pint_ness))) as bayes_prob,
	now() as lastupdate,
	count(*) as num_toks
from @@SCHEMA@@.get_prg_toks($1, (select longest_prg_title
	from @@SCHEMA@@.prg_long_title plt
	where plt.prg_oid = $1), '''') pt join @@SCHEMA@@.tok_stats_mv ts
	using (tok_src, tok_name, tok_subname, tok_value)
group by pt.prg_oid, pt.prg_title, ts.pint_class
order by bayes_prob desc, count(*) desc
'
    LANGUAGE sql STABLE STRICT;

CREATE FUNCTION compute_bayes_class(bigint) RETURNS bayes_cache
    AS 'select * from @@SCHEMA@@.compute_bayes_class_all($1)
order by bayes_prob desc, num_toks desc
limit 1;
'
    LANGUAGE sql STABLE STRICT;

CREATE VIEW shows_grid AS
    SELECT s.prg_oid, s.prg_start, s.prg_stop, blk_start(s.prg_start) AS blk_start, blk_end(s.prg_stop) AS blk_end, s.prg_title, s.chan_oid, s.chan_number, s.chan_name, pi.pint_class, COALESCE(bc.pint_class, get_bayes_class(s.prg_oid)) AS bayes_class FROM ((shows s LEFT JOIN bayes_cache bc USING (prg_oid)) LEFT JOIN prg_interest pi ON ((s.prg_title = pi.prg_title))) ORDER BY s.chan_number, s.chan_name, s.prg_start;

CREATE FUNCTION shows_grid_then(timestamp with time zone) RETURNS SETOF shows_grid
    AS 'SELECT s.prg_oid, s.prg_start, s.prg_stop,
	@@SCHEMA@@.blk_startx(s.prg_start, $1) AS blk_start,
	@@SCHEMA@@.blk_endx(s.prg_stop, $1) AS blk_end,
	s.prg_title, s.chan_oid, s.chan_number, s.chan_name,
	pi.pint_class, coalesce(bc.pint_class, @@SCHEMA@@.get_bayes_class(s.prg_oid)) as bayes_class
FROM @@SCHEMA@@.shows s
	left join @@SCHEMA@@.bayes_cache bc using (prg_oid)
	left join @@SCHEMA@@.prg_interest pi on s.prg_title = pi.prg_title
WHERE s.prg_start >= ($1 - ''08:00:00''::interval)
	AND s.prg_start < @@SCHEMA@@.grid_endx($1)
	AND s.prg_stop > @@SCHEMA@@.round_5min($1)
	AND s.prg_stop <= (@@SCHEMA@@.grid_endx($1) + ''08:00:00''::interval)
ORDER BY s.chan_number, s.chan_name, s.prg_start;'
    LANGUAGE sql STABLE;

CREATE INDEX ix_prg_start_stop ON programmes USING btree (prg_start, prg_stop);

ALTER TABLE programmes CLUSTER ON ix_prg_start_stop;

CREATE INDEX ix_cicons_chan_oid ON chan_icons USING btree (chan_oid);

CREATE INDEX ix_patt_search ON prg_attrs USING btree (prg_oid, patt_name, patt_value);

ALTER TABLE prg_attrs CLUSTER ON ix_patt_search;

CREATE INDEX ix_catt_best_name ON chan_attrs USING btree (chan_oid, catt_value) WHERE (((catt_name)::text = 'display-name'::text) AND (catt_value ~ '^[0-9]+ [A-Z]+$'::text));

CREATE INDEX ix_catt_search ON chan_attrs USING btree (chan_oid, catt_name, catt_value);

ALTER TABLE chan_attrs CLUSTER ON ix_catt_search;

CREATE INDEX ix_tsmv ON tok_stats_mv USING btree (tok_src, tok_name, tok_subname, tok_value, pint_class);

ALTER TABLE tok_stats_mv CLUSTER ON ix_tsmv;

CREATE INDEX ix_tspmv ON tok_sum_pint_mv USING btree (tok_src, tok_name, tok_subname, tok_value);

ALTER TABLE ONLY cv_interest
    ADD CONSTRAINT cv_interest_pkey PRIMARY KEY (pint_class);

ALTER TABLE ONLY prg_interest
    ADD CONSTRAINT prg_interest_pkey PRIMARY KEY (prg_title);

ALTER TABLE ONLY bayes_toks
    ADD CONSTRAINT ux_tok_uniq UNIQUE (tok_src, tok_name, tok_subname, tok_value, prg_oid);

ALTER TABLE ONLY channels
    ADD CONSTRAINT channels_pkey PRIMARY KEY (chan_oid);

ALTER TABLE channels CLUSTER ON channels_pkey;

ALTER TABLE ONLY channels
    ADD CONSTRAINT channels_chan_id_key UNIQUE (chan_id);

ALTER TABLE ONLY chan_attrs
    ADD CONSTRAINT channel_attrs_pkey PRIMARY KEY (catt_oid);

ALTER TABLE ONLY icons
    ADD CONSTRAINT icons_pkey PRIMARY KEY (icn_oid);

ALTER TABLE ONLY programmes
    ADD CONSTRAINT programmes_pkey PRIMARY KEY (prg_oid);

ALTER TABLE ONLY prg_attrs
    ADD CONSTRAINT prg_attrs_pkey PRIMARY KEY (patt_oid);

ALTER TABLE ONLY bayes_cache
    ADD CONSTRAINT bayes_cache_pkey PRIMARY KEY (prg_oid);

ALTER TABLE ONLY prg_interest
    ADD CONSTRAINT pint_cv_ref FOREIGN KEY (pint_class) REFERENCES cv_interest(pint_class) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY bayes_toks
    ADD CONSTRAINT "$1" FOREIGN KEY (pint_class) REFERENCES cv_interest(pint_class) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY programmes
    ADD CONSTRAINT prg_chan_ref FOREIGN KEY (chan_oid) REFERENCES channels(chan_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY prg_icons
    ADD CONSTRAINT prg_icon_prg_ref FOREIGN KEY (prg_oid) REFERENCES programmes(prg_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY prg_attrs
    ADD CONSTRAINT patt_prg_ref FOREIGN KEY (prg_oid) REFERENCES programmes(prg_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY patt_icons
    ADD CONSTRAINT patt_icon_patt_ref FOREIGN KEY (patt_oid) REFERENCES prg_attrs(patt_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY chan_icons
    ADD CONSTRAINT chan_icon_chan_ref FOREIGN KEY (chan_oid) REFERENCES channels(chan_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY chan_attrs
    ADD CONSTRAINT catt_chan_ref FOREIGN KEY (chan_oid) REFERENCES channels(chan_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY bayes_cache
    ADD CONSTRAINT "$1" FOREIGN KEY (prg_oid) REFERENCES programmes(prg_oid) ON UPDATE RESTRICT ON DELETE CASCADE;

COMMENT ON TABLE cv_interest IS 'controlled vocabulary for programme interest classification';

COMMENT ON TABLE prg_interest IS 'programme interest classification
(by programme title)';

COMMENT ON TABLE bayes_toks IS 'full token data for bayes learning';

COMMENT ON COLUMN bayes_toks.tok_value IS 'token value, not att value';

COMMENT ON FUNCTION longer(character varying, character varying) IS 'Returns the longer of two input strings';

COMMENT ON AGGREGATE longest(character varying) IS 'Chooses the string-wise longset value from a set';

COMMENT ON FUNCTION round_5min(timestamp with time zone) IS 'Rounds a time to the previous five minute mark';

COMMENT ON FUNCTION now_5min() IS 'Rounds current time to previous 5 minute mark';

COMMENT ON FUNCTION grid_end() IS 'Returns the time at which the current grid should end';

COMMENT ON FUNCTION roundup_5min(timestamp with time zone) IS 'Rounds a time to the next five minute mark';

COMMENT ON FUNCTION blk_start(timestamp with time zone) IS 'Returns the ''programming block'' start for a given time - the time rounded to a five minute mark, shifted forwards to be at least now';

COMMENT ON FUNCTION blk_end(timestamp with time zone) IS 'Returns the ''programming block'' end time - the time rounted to a five minute mark, truncated down to be not more than the grid end for now.';

COMMENT ON TABLE channels IS 'channels (mostly just ids)';

COMMENT ON TABLE chan_attrs IS 'channel attributes';

COMMENT ON TABLE icons IS 'base icons table from which other tables inherit';

COMMENT ON TABLE chan_icons IS 'channel icons';

COMMENT ON TABLE programmes IS 'programmes
(most info is actually in prg_attrs)';

COMMENT ON TABLE prg_icons IS 'programme icons
(rarely used)';

COMMENT ON TABLE prg_attrs IS 'programme attributes';

COMMENT ON TABLE patt_icons IS 'icons for programme attributes
(rarely used)';

COMMENT ON VIEW chan_best_name IS 'Best name for a channel to present to user';

COMMENT ON FUNCTION grid_endx(timestamp with time zone) IS 'Returns the time at which a grid should end when it starts at the passed time.';

COMMENT ON FUNCTION blk_startx(timestamp with time zone, timestamp with time zone) IS 'The programming block start for a given time and grid start';

COMMENT ON FUNCTION prgs_then(timestamp with time zone) IS 'Fetches programmes active in a grid starting at the given time';

COMMENT ON FUNCTION prg_attrs_then(timestamp with time zone) IS 'Fetches programme attrs for prgs on during a grid block starting at the given time';

COMMENT ON FUNCTION agg_prod_next(double precision, double precision) IS 'next state function for prod() aggregate';

COMMENT ON FUNCTION tokenizer(text) IS 'return all the tokens from a string';

COMMENT ON TABLE tok_counts_val_pint_mv IS 'counts of occurrences of (token, pint_class)';

COMMENT ON TABLE tok_counts_val_mv IS 'counts of occurrences of tokens';

COMMENT ON TABLE tok_counts_pint_mv IS 'counts of distinct prg_oids under each pint_class';

COMMENT ON TABLE tok_counts_full_mv IS 'join of all the token counts';

COMMENT ON FUNCTION get_bayes_class(bigint) IS 'bayes_class = get_bayes_class(prg_oid) -- fetches bayes classification for a prg_oid, using and maintaining cache';

COMMENT ON FUNCTION get_prg_toks(bigint, character varying, character varying) IS 'setof bayes_toks = get_prg_toks(prg_oid, prg_title, pint_class) -- produces all the bayes_toks for a prg_oid, using the prg_title and pint_class given';

COMMENT ON FUNCTION bayes_learn_prg(bigint, character varying, character varying) IS 'token_count = bayes_learn_prg(prg_oid, prg_title, pint_class) -- computes and stores bayes tokens for a prg_oid, pint_class, prg_title.
if pint_class is null, unlearns (removes tokens), if it is '''', does a lookup to find it.
if prg title is null or '''', does a lookup to find it.';

COMMENT ON FUNCTION min(anyelement, anyelement) IS 'returns the minimum of two elements';

COMMENT ON FUNCTION max(anyelement, anyelement) IS 'returns the maximum of two elements';

COMMENT ON TABLE tok_stats_mv IS 'pint_class-ness for each (token, pint_class)';

COMMENT ON FUNCTION blk_endx(timestamp with time zone, timestamp with time zone) IS 'The programming block end for a time and a grid start';

COMMENT ON VIEW prgs_now IS 'Programmes that are showing right now';

COMMENT ON FUNCTION update_bayes_stats() IS 'Update all bayes statistics materialized view tables';

COMMENT ON VIEW tok_stats IS 'pint_class-ness for each (token, pint_class)';

COMMENT ON VIEW prg_long_title IS 'Longest title for a show';

COMMENT ON VIEW shows IS 'Shows - program with title and channel info';

COMMENT ON VIEW chan_quality IS 'Attempts to estimate the overall quality of a channel by tallying the ratings of the shows it airs';

COMMENT ON TABLE bayes_cache IS 'cache for computed bayseian classification';

COMMENT ON FUNCTION compute_bayes_class_all(bigint) IS 'setof bayes_cache%rowtype = compute_bayes_cache(prg_oid) -- computes the bayes probability for each pint_class for a prg_oid';

COMMENT ON FUNCTION compute_bayes_class(bigint) IS 'bayes_cache%rowtype = compute_bayes_cache(prg_oid) -- computes the value for the bayes_cache row for a prg_oid';

COMMENT ON VIEW shows_grid IS 'Shows with grid information added';

COMMENT ON FUNCTION shows_grid_then(timestamp with time zone) IS 'Fetches a table like active_shows_grid, but for a grid starting other than now';

SET client_encoding = 'SQL_ASCII';
SET check_function_bodies = false;

SET search_path = @@SCHEMA@@, pg_catalog;
COPY cv_interest (pint_class, pint_order, weight) FROM stdin;
good	1	2
normal	2	0
crap	3	0
bad	4	-1
\.
